@@ -12,9 +12,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * The Sushi project hereby grant permission for non-gpl compatible GStreamer
  * plugins to be used and distributed together with GStreamer and Sushi. This
@@ -25,6 +23,7 @@
  *
  */
 
+const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
 const GObject = imports.gi.GObject;
 const Gdk = imports.gi.Gdk;
@@ -44,6 +43,7 @@ const Mainloop = imports.mainloop;
 const MimeHandler = imports.ui.mimeHandler;
 const Constants = imports.util.constants;
 const SpinnerBox = imports.ui.spinnerBox;
+const Utils = imports.ui.utils;
 
 const Sushi = imports.gi.Sushi;
 
@@ -55,31 +55,50 @@ MainWindow.prototype = {
     _init : function(args) {
         args = args || {};
 
+        this._background = null;
+        this._pendingRenderer = null;
+        this._renderer = null;
+        this._texture = null;
+        this._toolbarActor = null;
+        this._fullScreenId = 0;
+        this._toolbarId = 0;
+        this._unFullScreenId = 0;
+
         this._mimeHandler = new MimeHandler.MimeHandler();
 
         this._application = args.application;
         this._createGtkWindow();
         this._createClutterEmbed();
 
-        this._connectStageSignals();
 	this.file = null;
     },
 
     _createGtkWindow : function() {
+        this._settings = new Gio.Settings({ schema_id: 'org.gnome.sushi' });
+        this._clientDecorated = this._settings.get_boolean('client-decoration');
+
         this._gtkWindow = new Gtk.Window({ type: Gtk.WindowType.TOPLEVEL,
                                            focusOnMap: true,
-                                           decorated: false,
+                                           decorated: !this._clientDecorated,
                                            hasResizeGrip: false,
                                            skipPagerHint: true,
                                            skipTaskbarHint: true,
                                            windowPosition: Gtk.WindowPosition.CENTER,
-                                           gravity: Gdk.Gravity.CENTER });
+                                           gravity: Gdk.Gravity.CENTER,
+                                           application: this._application });
 
         let screen = Gdk.Screen.get_default();
         this._gtkWindow.set_visual(screen.get_rgba_visual());
 
-        this._gtkWindow.connect("delete-event",
+        this._gtkWindow.connect('delete-event',
                                 Lang.bind(this, this._onWindowDeleteEvent));
+        this._gtkWindow.connect('realize', Lang.bind(this,
+            function() {
+                // don't support maximize and minimize
+                this._gtkWindow.get_window().set_functions(Gdk.WMFunction.MOVE |
+                                                           Gdk.WMFunction.RESIZE |
+                                                           Gdk.WMFunction.CLOSE);
+            }));
     },
 
     _createClutterEmbed : function() {
@@ -96,22 +115,20 @@ MainWindow.prototype = {
                                                   green: 0,
                                                   blue: 0,
                                                   alpha: 255 }));
-        this._mainGroup =  new Clutter.Group();
+
+        this._mainLayout = new Clutter.BinLayout();
+        this._mainGroup = new Clutter.Actor({ layout_manager: this._mainLayout });
+        this._mainGroup.add_constraint(
+            new Clutter.BindConstraint({ coordinate: Clutter.BindCoordinate.SIZE,
+                                         source: this._stage }));
         this._stage.add_actor(this._mainGroup);
-
-        this._titleGroup =  new Clutter.Group();
-        this._stage.add_actor(this._titleGroup);
-
         this._mainGroup.set_opacity(0);
-        this._titleGroup.set_opacity(0);
-    },
 
-    _connectStageSignals : function() {
-        this._stage.connect("key-press-event",
+        this._stage.connect('key-press-event',
                             Lang.bind(this, this._onStageKeyPressEvent));
-        this._stage.connect("button-press-event",
+        this._stage.connect('button-press-event',
                             Lang.bind(this, this._onButtonPressEvent));
-        this._stage.connect("motion-event",
+        this._stage.connect('motion-event',
                             Lang.bind(this, this._onMotionEvent));
     },
 
@@ -125,14 +142,8 @@ MainWindow.prototype = {
                                                        green: 0,
                                                        blue: 0,
                                                        alpha: 255 }));
-        this._background.add_constraint(
-            new Clutter.BindConstraint({ source: this._stage,
-                                         coordinate: Clutter.BindCoordinate.POSITION }));
-        this._background.add_constraint(
-            new Clutter.BindConstraint({ source: this._stage,
-                                         coordinate: Clutter.BindCoordinate.SIZE }));
 
-        this._mainGroup.add_actor(this._background);
+        this._mainLayout.add(this._background, Clutter.BinAlignment.FILL, Clutter.BinAlignment.FILL);
         this._background.lower_bottom();
     },
 
@@ -140,16 +151,19 @@ MainWindow.prototype = {
         if (this._background)
             return;
 
-        this._background = Sushi.create_rounded_background();
+        if (this._clientDecorated) {
+            this._background = Sushi.create_rounded_background();
+        } else {
+            this._background = new Clutter.Rectangle();
+            this._background.set_color(new Clutter.Color({ red: 0,
+                                                           green: 0,
+                                                           blue: 0,
+                                                           alpha: 255 }));
+        }
+
         this._background.set_opacity(Constants.VIEW_BACKGROUND_OPACITY);
-        this._background.add_constraint(
-            new Clutter.BindConstraint({ source: this._stage,
-                                         coordinate: Clutter.BindCoordinate.POSITION }));
-        this._background.add_constraint(
-            new Clutter.BindConstraint({ source: this._stage,
-                                         coordinate: Clutter.BindCoordinate.SIZE }));
+        this._mainLayout.add(this._background, Clutter.BinAlignment.FILL, Clutter.BinAlignment.FILL);
 
-        this._mainGroup.add_actor(this._background);
         this._background.lower_bottom();
     },
 
@@ -187,7 +201,7 @@ MainWindow.prototype = {
             return false;
         }
 
-        let root_coords = 
+        let root_coords =
             this._gtkWindow.get_window().get_root_coords(win_coords[0],
                                                          win_coords[1]);
 
@@ -212,9 +226,10 @@ MainWindow.prototype = {
     _getTextureSize : function() {
         let screenSize = [ this._gtkWindow.get_window().get_width(),
                            this._gtkWindow.get_window().get_height() ];
+        let yPadding = this._clientDecorated ? Constants.VIEW_PADDING_Y : 0;
 
         let availableWidth = this._isFullScreen ? screenSize[0] : Constants.VIEW_MAX_W - 2 * Constants.VIEW_PADDING_X;
-        let availableHeight = this._isFullScreen ? screenSize[1] : Constants.VIEW_MAX_H - Constants.VIEW_PADDING_Y;
+        let availableHeight = this._isFullScreen ? screenSize[1] : Constants.VIEW_MAX_H - yPadding;
 
         let textureSize = this._renderer.getSizeForAllocation([availableWidth, availableHeight], this._isFullScreen);
 
@@ -224,13 +239,14 @@ MainWindow.prototype = {
     _getWindowSize : function() {
         let textureSize = this._getTextureSize();
         let windowSize = textureSize;
+        let yPadding = this._clientDecorated ? Constants.VIEW_PADDING_Y : 0;
 
         if (textureSize[0] < (Constants.VIEW_MIN - 2 * Constants.VIEW_PADDING_X) &&
-            textureSize[1] < (Constants.VIEW_MIN - Constants.VIEW_PADDING_Y)) {
+            textureSize[1] < (Constants.VIEW_MIN - yPadding)) {
             windowSize = [ Constants.VIEW_MIN, Constants.VIEW_MIN ];
         } else if (!this._isFullScreen) {
             windowSize = [ windowSize[0] + 2 * Constants.VIEW_PADDING_X,
-                           windowSize[1] + Constants.VIEW_PADDING_Y ];
+                           windowSize[1] + yPadding ];
         }
 
         return windowSize;
@@ -248,7 +264,7 @@ MainWindow.prototype = {
         }
 
         if (yFactor == 0) {
-            if (this._isFullScreen && 
+            if (this._isFullScreen &&
                (textureSize[0] > textureSize[1]))
                 yFactor = 0.52;
             else
@@ -275,7 +291,7 @@ MainWindow.prototype = {
             if (this._renderer.clear)
                 this._renderer.clear();
 
-            delete this._renderer;
+            this._renderer = null;
         }
 
         /* create a temporary spinner renderer, that will timeout and show itself
@@ -285,7 +301,7 @@ MainWindow.prototype = {
         this._renderer.startTimeout();
 
         file.query_info_async
-        (Gio.FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME + "," +
+        (Gio.FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME + ',' +
          Gio.FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
          Gio.FileQueryInfoFlags.NONE,
          GLib.PRIORITY_DEFAULT, null,
@@ -300,6 +316,7 @@ MainWindow.prototype = {
                             this._pendingRenderer.prepare(file, this, Lang.bind(this, this._onRendererPrepared));
                         } catch(e) {
                             /* FIXME: report the error */
+                            logError(e, 'Error calling prepare() on viewer');
                         }}));
     },
 
@@ -308,7 +325,7 @@ MainWindow.prototype = {
         this._renderer.destroy();
 
         this._renderer = this._pendingRenderer;
-        delete this._pendingRenderer;
+        this._pendingRenderer = null;
 
         /* generate the texture and toolbar for the new renderer */
         this._createTexture();
@@ -318,24 +335,18 @@ MainWindow.prototype = {
     _createTexture : function() {
         if (this._texture) {
             this._texture.destroy();
-            delete this._texture;
+            this._texture = null;
         }
 
         this._texture = this._renderer.render();
-
-        this._textureXAlign = 
-            new Clutter.AlignConstraint({ source: this._stage,
-                                          factor: 0.5 });
         this._textureYAlign =
             new Clutter.AlignConstraint({ source: this._stage,
                                           factor: 0.5,
-                                          "align-axis": Clutter.AlignAxis.Y_AXIS })
-
-        this._texture.add_constraint(this._textureXAlign);
+                                          align_axis: Clutter.AlignAxis.Y_AXIS });
         this._texture.add_constraint(this._textureYAlign);
 
         this.refreshSize();
-        this._mainGroup.add_actor(this._texture);
+        this._mainGroup.add_child(this._texture);
     },
 
     /**************************************************************************
@@ -343,11 +354,11 @@ MainWindow.prototype = {
      **************************************************************************/
     _onStageUnFullScreen : function() {
         this._stage.disconnect(this._unFullScreenId);
-        delete this._unFullScreenId;
+        this._unFullScreenId = 0;
 
 	/* We want the alpha background back now */
         this._background.destroy();
-        delete this._background;
+        this._background = null;
         this._createAlphaBackground();
 
         this._textureYAlign.factor = this._savedYFactor;
@@ -359,12 +370,12 @@ MainWindow.prototype = {
         Tweener.addTween(this._mainGroup,
                          { opacity: 255,
                            time: 0.15,
-                           transition: 'easeOutQuad',
+                           transition: 'easeOutQuad'
                          });
         Tweener.addTween(this._titleGroup,
                          { opacity: 255,
                            time: 0.15,
-                           transition: 'easeOutQuad',
+                           transition: 'easeOutQuad'
                          });
     },
 
@@ -376,11 +387,11 @@ MainWindow.prototype = {
             this._removeToolbarTimeout();
         }
 
-        /* wait for the next stage allocation to fade in the texture 
+        /* wait for the next stage allocation to fade in the texture
          * and background again.
          */
         this._unFullScreenId =
-            this._stage.connect("notify::allocation",
+            this._stage.connect('notify::allocation',
                                 Lang.bind(this, this._onStageUnFullScreen));
 
         /* quickly fade out everything,
@@ -399,11 +410,11 @@ MainWindow.prototype = {
 
     _onStageFullScreen : function() {
         this._stage.disconnect(this._fullScreenId);
-        delete this._fullScreenId;
+        this._fullScreenId = 0;
 
         /* We want a solid black background */
         this._background.destroy();
-        delete this._background;
+        this._background = null;
 	this._createSolidBackground();
 
 	/* Fade in everything but the title */
@@ -447,11 +458,11 @@ MainWindow.prototype = {
             this._removeToolbarTimeout();
         }
 
-        /* wait for the next stage allocation to fade in the texture 
+        /* wait for the next stage allocation to fade in the texture
          * and background again.
          */
         this._fullScreenId =
-            this._stage.connect("notify::allocation",
+            this._stage.connect('notify::allocation',
                                 Lang.bind(this, this._onStageFullScreen));
 
         /* quickly fade out everything,
@@ -478,8 +489,9 @@ MainWindow.prototype = {
      **************************************************************************/
     _createToolbar : function() {
         if (this._toolbarActor) {
+            this._removeToolbarTimeout();
             this._toolbarActor.destroy();
-            delete this._toolbarActor;
+            this._toolbarActor = null;
         }
 
         if (this._renderer.createToolbar)
@@ -488,35 +500,28 @@ MainWindow.prototype = {
         if (!this._toolbarActor)
             return;
 
+        Utils.alphaGtkWidget(this._toolbarActor.get_widget());
+
         this._toolbarActor.set_reactive(true);
         this._toolbarActor.set_opacity(0);
-        this._mainGroup.add_actor(this._toolbarActor);
 
-        this._toolbarActor.add_constraint(
-            new Clutter.AlignConstraint({ source: this._stage,
-                                          factor: 0.5 }));
+        this._toolbarActor.margin_bottom = Constants.TOOLBAR_SPACING;
+        this._toolbarActor.margin_left = Constants.TOOLBAR_SPACING;
+        this._toolbarActor.margin_right = Constants.TOOLBAR_SPACING;
 
-        let yConstraint = 
-            new Clutter.BindConstraint({ source: this._stage,
-                                         coordinate: Clutter.BindCoordinate.Y,
-                                         offset: this._stage.height - Constants.TOOLBAR_SPACING });
-        this._toolbarActor.add_constraint(yConstraint);
-
-        this._stage.connect("notify::height",
-                            Lang.bind(this, function() {
-                                yConstraint.set_offset(this._stage.height - Constants.TOOLBAR_SPACING);
-                            }));
+        this._mainLayout.add(this._toolbarActor,
+                             Clutter.BinAlignment.CENTER, Clutter.BinAlignment.END);
     },
 
     _removeToolbarTimeout: function() {
-        Mainloop.source_remove(this._toolbarId);
-        delete this._toolbarId;
+        if (this._toolbarId != 0) {
+            Mainloop.source_remove(this._toolbarId);
+            this._toolbarId = 0;
+        }
     },
 
     _resetToolbar : function() {
-        if (this._toolbarId) {
-            this._removeToolbarTimeout();
-        } else {
+        if (this._toolbarId == 0) {
             Tweener.removeTweens(this._toolbarActor);
 
             this._toolbarActor.raise_top();
@@ -525,17 +530,18 @@ MainWindow.prototype = {
             Tweener.addTween(this._toolbarActor,
                              { opacity: 200,
                                time: 0.1,
-                               transition: 'easeOutQuad',
+                               transition: 'easeOutQuad'
                              });
         }
 
+        this._removeToolbarTimeout();
         this._toolbarId = Mainloop.timeout_add(1500,
                                                Lang.bind(this,
                                                          this._onToolbarTimeout));
     },
 
     _onToolbarTimeout : function() {
-        delete this._toolbarId;
+        this._toolbarId = 0;
         Tweener.addTween(this._toolbarActor,
                          { opacity: 0,
                            time: 0.25,
@@ -549,56 +555,57 @@ MainWindow.prototype = {
      ************************ titlebar helpers ********************************
      **************************************************************************/
     _createTitle : function() {
-        if (this._titleLabel) {
-            this._titleActor.raise_top();
-            this._quitActor.raise_top();
+        if (this._titleGroup) {
+            this._titleGroup.raise_top();
             return;
         }
 
-        this._titleLabel = new Gtk.Label({ label: "",
-					   ellipsize: Pango.EllipsizeMode.END });
-        this._titleLabel.get_style_context().add_class("np-decoration");
-        
+        this._titleGroupLayout = new Clutter.BoxLayout();
+        this._titleGroup =  new Clutter.Box({ layout_manager: this._titleGroupLayout,
+                                              opacity: 0 });
+        this._stage.add_actor(this._titleGroup);
+
+        this._titleGroup.add_constraint(
+            new Clutter.BindConstraint({ source: this._stage,
+                                         coordinate: Clutter.BindCoordinate.WIDTH }));
+
+        // if we don't draw client decorations, just create an empty actor
+        if (!this._clientDecorated)
+            return;
+
+        this._titleLabel = new Gtk.Label({ label: '',
+					   ellipsize: Pango.EllipsizeMode.END,
+                                           margin: 6 });
+        this._titleLabel.get_style_context().add_class('np-decoration');
+
         this._titleLabel.show();
         this._titleActor = new GtkClutter.Actor({ contents: this._titleLabel });
-        this._titleActor.add_constraint(
-            new Clutter.AlignConstraint({ source: this._stage,
-                                          align_axis: Clutter.AlignAxis.X_AXIS,
-                                          factor: 0.5 }));
+        Utils.alphaGtkWidget(this._titleActor.get_widget());
 
-        this._quitButton = 
-            new Gtk.Button({ image: new Gtk.Image ({ "icon-size": Gtk.IconSize.MENU,
-                                                     "icon-name": "window-close-symbolic" })});
-        this._quitButton.get_style_context().add_class("np-decoration");
+        this._quitButton =
+            new Gtk.Button({ image: new Gtk.Image ({ icon_size: Gtk.IconSize.MENU,
+                                                     icon_name: 'window-close-symbolic' })});
+        this._quitButton.get_style_context().add_class('np-decoration');
         this._quitButton.show();
 
-        this._quitButton.connect("clicked",
+        this._quitButton.connect('clicked',
                                  Lang.bind(this,
                                            this._clearAndQuit));
 
         this._quitActor = new GtkClutter.Actor({ contents: this._quitButton });
+        Utils.alphaGtkWidget(this._quitActor.get_widget());
         this._quitActor.set_reactive(true);
-        this._quitActor.add_constraint(
-            new Clutter.AlignConstraint({ source: this._stage,
-                                          factor: 1.0 }));
 
-        this._titleActor.add_constraint(
-            new Clutter.SnapConstraint({ source: this._stage,
-                                         from_edge: Clutter.SnapEdge.LEFT,
-                                         to_edge: Clutter.SnapEdge.LEFT,
-                                         offset: 6 }));
-        this._titleActor.add_constraint(
-            new Clutter.SnapConstraint({ source: this._quitActor,
-                                         from_edge: Clutter.SnapEdge.RIGHT,
-                                         to_edge: Clutter.SnapEdge.LEFT,
-                                         offset: -6 }));
-        this._titleActor.add_constraint(
-            new Clutter.BindConstraint({ source: this._stage,
-                                         coordinate: Clutter.BindCoordinate.Y,
-                                         offset: 6 }));
+        let hidden = new Clutter.Actor();
+        let size = this._quitButton.get_preferred_size()[1];
+        hidden.set_size(size.width, size.height);
 
-        this._titleGroup.add_actor(this._titleActor);
-        this._titleGroup.add_actor(this._quitActor);
+        this._titleGroupLayout.pack(hidden, false, false, false,
+                                    Clutter.BoxAlignment.START, Clutter.BoxAlignment.START);
+        this._titleGroupLayout.pack(this._titleActor, true, true, false,
+                                    Clutter.BoxAlignment.CENTER, Clutter.BoxAlignment.START);
+        this._titleGroupLayout.pack(this._quitActor, false, false, false,
+                                    Clutter.BoxAlignment.END, Clutter.BoxAlignment.START);
     },
 
     /**************************************************************************
@@ -621,14 +628,12 @@ MainWindow.prototype = {
     },
 
     _fadeOutWindow : function() {
-        if (this._toolbarId) {
-            this._removeToolbarTimeout();
-        }
+        this._removeToolbarTimeout();
 
         Tweener.addTween(this._titleGroup,
                          { opacity: 0,
                            time: 0.15,
-                           transition: 'easeOutQuad',
+                           transition: 'easeOutQuad'
                          });
 
         Tweener.addTween(this._mainGroup,
@@ -646,7 +651,7 @@ MainWindow.prototype = {
         if (this._renderer.clear)
             this._renderer.clear();
 
-        this._application.quit();
+        this._gtkWindow.destroy();
     },
 
     /**************************************************************************
@@ -657,6 +662,8 @@ MainWindow.prototype = {
         this._gtkWindow.realize();
         this._gtkWindow.get_window().set_transient_for(this._parent);
         this._gtkWindow.show_all();
+
+        this._gtkWindow.get_window().move_to_current_desktop();
     },
 
     setFile : function(file) {
@@ -671,7 +678,10 @@ MainWindow.prototype = {
     },
 
     setTitle : function(label) {
-        this._titleLabel.set_label(label);
+        if (this._clientDecorated)
+            this._titleLabel.set_label(label);
+
+        this._gtkWindow.set_title(label);
     },
 
     refreshSize : function() {
